import networkx as nx
from .egfuncs import *
import plotly.graph_objs as go
import copy

class Edge:
    def __init__(self, G, dim):
        self.G = G
        self.dim = dim
        self.settings = []
        self.default_color = 'black'
        self.cmin = None
        self.cmax = None
        self.wmin = None
        self.wmax = None
        self.default_min_width = 0.5
        self.default_max_width = 5

    def load(self, df, from_col, to_col):
        for index, row in df.iterrows():
            f = row[from_col]
            t = row[to_col]
            mydict = row.to_dict()
            mydict.pop(from_col, None)
            mydict.pop(to_col, None)
            self.add(f, t, mydict)

    def add(self, from_node, to_node, attributes=None):
        """
           Add edges to the graph.
           Params:
               from_node: initial node
               to_node: destination node
               attributes (optional): dicionary with keys and its corresponding values
        """
        self.G.add_edge(from_node, to_node)
        if attributes is not None:
            for k, v in attributes.items():
                self.G.edges[(from_node,to_node)][k]=v

    def get_min_max(self, attribute):
        try:
            min = 10e10
            max = -10e10
            for g in self.G.edges:
                if self.G.edges[g][attribute] < min:
                    min = self.G.edges[g][attribute]
                if self.G.edges[g][attribute] > max:
                    max = self.G.edges[g][attribute]
        except:
            raise
        return min, max

    def get_cmin_cmax(self, attribute):
        self.cmin, self.cmax = self.get_min_max(attribute)

    def get_wmin_wmax(self, attribute):
        self.wmin, self.wmax = self.get_min_max(attribute)

    def get_2d_trace(self, color_col, width_col):
        settings = []
        trace_default = go.Scatter(
            x=[],
            y=[],
            line=dict(
                width=self.default_min_width,
                color=self.default_color,
            ),
            hoverinfo='none',
            mode='lines'
        )
        trace = copy.deepcopy(trace_default)
        for edge in self.G.edges():
            x0, y0 = self.G.node[edge[0]]['pos']
            x1, y1 = self.G.node[edge[1]]['pos']
            trace['x'] = tuple([x0, x1, None])
            trace['y'] = tuple([y0, y1, None])
            trace['name'] = edge[0] + '-' + edge[1]
            if color_col == None:
                color = self.default_color
            else:
                nvalue = self.G.edges[edge][color_col]
                pvalue = (nvalue - self.cmin)/(self.cmax-self.cmin)
                color = get_color(pvalue)[1]

            if width_col == None:
                width = self.default_min_width
            else:
                nvalue = self.G.edges[edge][width_col]
                pvalue = self.default_min_width + (nvalue - self.wmin)/(self.wmax-self.wmin)*self.default_max_width
                width = pvalue
            trace['line'] = dict(width=width,color=color)
            settings.append(copy.deepcopy(trace))
        return settings

    def get_3d_trace(self, color_col, width_col):
        settings = []
        trace_default = go.Scatter3d(
            x=[],
            y=[],
            z=[],
            line=dict(
                width=self.default_min_width,
                color=self.default_color,
            ),
            hoverinfo='none',
            mode='lines'
        )
        trace = copy.deepcopy(trace_default)
        for edge in self.G.edges():
            x0, y0, z0 = self.G.node[edge[0]]['pos']
            x1, y1, z1 = self.G.node[edge[1]]['pos']

            trace['x'] = tuple([x0, x1, None])
            trace['y'] = tuple([y0, y1, None])
            trace['z'] = tuple([z0, z1, None])
            trace['name'] = edge[0] + '-' + edge[1]
            if color_col == None:
                color = self.default_color
            else:
                nvalue = self.G.edges[edge][color_col]
                pvalue = (nvalue - self.cmin)/(self.cmax-self.cmin)
                color = get_color(pvalue)[1]

            if width_col == None:
                width = self.default_min_width
            else:
                nvalue = self.G.edges[edge][width_col]
                pvalue = self.default_min_width + (nvalue - self.wmin)/(self.wmax-self.wmin)*self.default_max_width
                width = pvalue
            trace['line'] = dict(width=width,color=color)
            settings.append(copy.deepcopy(trace))
        return settings

    def set_attribute(self, color_col, width_col):
        if color_col is not None:
            self.get_cmin_cmax(color_col)
        if width_col is not None:
            self.get_wmin_wmax(width_col)
        if self.dim == 3:
            self.settings = self.get_3d_trace(color_col, width_col)
        else:
            self.settings = self.get_2d_trace(color_col, width_col)



from matplotlib import cm
import matplotlib
import numpy as np

def iterable(obj):
    if type(obj) == 'str':
        return False
    try:
        iter(obj)
        return True
    except TypeError:
        return False

def get_color(n):
    magma_cmap = cm.get_cmap('magma')
    norm = matplotlib.colors.Normalize(vmin=0, vmax=255)
    magma_rgb = []
    for i in range(0, 255):
       k = matplotlib.colors.colorConverter.to_rgb(magma_cmap(norm(i)))
       magma_rgb.append(k)
    magma = matplotlib_to_plotly(magma_cmap, 255)
    if n < 0 or n > 1:
        print('Choose a number between 0 and 1')
        return
    return magma[int(n*254)]

def matplotlib_to_plotly(cmap, pl_entries):
    h = 1.0/(pl_entries-1)
    pl_colorscale = []
    for k in range(pl_entries):
        C = list(map(np.uint8, np.array(cmap(k*h)[:3])*255))
        pl_colorscale.append([k*h, 'rgb'+str((C[0], C[1], C[2]))])

    return pl_colorscale



from .egfuncs import *
import networkx as nx
import plotly.graph_objs as go
from .consts import *

class Node:
    def __init__(self, G, dim, dict=None):
        self.G = G
        self.dim = dim
        self.title = ''
        self.size = 5 # padronizar esse nome com o dos edges
        self.sizemin = self.size
        if dim == 3:
            self.sizemin=15
        self.sizemax = 50
        self.cmin = 10e1000
        self.cmax = -10e1000
        self.show_scale = False
        self.default_color = 'darkblue'
        self.annotations = []

    def load(self, df, id_col):
        for index, row in df.iterrows():
            v = row[id_col]
            mydict = row.to_dict()
            mydict.pop(id_col, None)
            self.add(v, mydict)

    def add(self, name, attributes=None):
        """
           Add nodes to the graph.
           Params:
               node_name: node id
               attributes (optional): dicionary with keys and its corresponding values
        """
        self.G.add_node(name)
        if attributes is not None:
            for k, v in attributes.items():
                self.G.nodes[name][k]=v

    def set_pos(self, layout, dim):
        layouts = {
            1: nx.circular_layout,
            2: nx.random_layout,
            3: nx.shell_layout,
            4: nx.spring_layout,
            5: nx.spectral_layout,
            6: nx.fruchterman_reingold_layout
        }
        layout_func = layouts[layout]
        self.pos = layout_func(self.G, dim = dim)
        self.G.add_nodes_from([(k[0], {'pos':k[1]}) for k in self.pos.items()])

    def get_size_params(self, node_size_col):
        if node_size_col is not None:
            self.size = []
            for node in self.G.nodes:
                self.size.append(self.G.nodes[node][node_size_col])
            self.sizeref = 2.*max(self.size)/(float(self.sizemax)**2)
        else:
            self.size = [1]*len(self.G.nodes)
            self.sizeref = self.sizemin

    def get_color_params(self, color_col):
        color = []
        if color_col is not None:
            for node in self.G.nodes:
                color.append(self.G.nodes[node][color_col])
        else:
            color = self.default_node_color
        return color

    def set_3d_settings(self):
        self.settings = go.Scatter3d(
            x=[],
            y=[],
            z=[],
            text=[],
            mode='markers',
            hoverinfo='text',
            marker=dict(
                showscale=self.show_scale,
                #colorscale='YlGnBu',
                colorscale=colorscale,
                reversescale=True,
        #        color=color,
                cmax = self.cmax,
                cmin = self.cmin,
                size = self.size,
                sizemode='area',
                sizeref=self.sizeref,
                sizemin=self.sizemin,
                colorbar=dict(
                    thickness=15,
                    title=self.title,
                    xanchor='left',
                    titleside='right'
                ),
                line=dict(width=2)))

    def set_2d_settings(self):
        self.settings = go.Scatter(
            x=[],
            y=[],
            text=[],
            mode='markers',
            hoverinfo='text',
            marker=dict(
                showscale=self.show_scale,
                #colorscale='YlGnBu',
                colorscale=colorscale,
                reversescale=True,
        #        color=color,
                cmax = self.cmax,
                cmin = self.cmin,
                size = self.size,
                sizemode='area',
                sizeref=self.sizeref,
                sizemin=self.sizemin,
                colorbar=dict(
                    thickness=15,
                    title=self.title,
                    xanchor='left',
                    titleside='right'
                ),
                line=dict(width=2)))

    def set_size_attribute(self, node_size_col, color_col):
        self.get_size_params(node_size_col)
        #color = self.get_color_params(color_col)
        if self.dim == 3:
            self.set_3d_settings()
        else:
            self.set_2d_settings()

    def set_2d_annotations(self):
        for node in self.G.nodes():
            x, y = self.G.node[node]['pos']
            self.settings['x'] += tuple([x])
            self.settings['y'] += tuple([y])
            self.annotations.append(
                dict(
                    x=x,
                    y=y+0.10,
                    xref='x',
                    yref='y',
                    text=node,
                    showarrow=False,
                    ax=0,
                    ay=20
                )
            )

    def set_3d_annotations(self):
        for node in self.G.nodes():
            x, y, z = self.G.node[node]['pos']
            self.settings['x'] += tuple([x])
            self.settings['y'] += tuple([y])
            self.settings['z'] += tuple([z])
            self.annotations.append(
                dict(
                    showarrow = False,
                    x=x,
                    y=y+0.10,
                    z=z,
                    text=node,
                    xanchor = "left",
                    xshift = 10,
                    yanchor='bottom',
                    font=dict(
                        size=14
                    )
                )
            )

    def set_annotations(self):
        if self.dim == 3:
            self.set_3d_annotations()
        else:
            self.set_2d_annotations()

    def set_2d_layout(self, title):
        self.set_annotations()
        self.layout = go.Layout(
            showlegend=False,
            annotations=self.annotations,
            title=title,
            xaxis=dict(
                    autorange=True,
                    showgrid=False,
                    zeroline=False,
                    showline=False,
                    ticks='',
                    showticklabels=False
                ),
            yaxis=dict(
                    autorange=True,
                    showgrid=False,
                    zeroline=False,
                    showline=False,
                    ticks='',
                    showticklabels=False
                ),
            titlefont=dict(size=16),
            hovermode='closest',
            margin=dict(b=20,l=5,r=5,t=40),
            scene = dict(
                xaxis=dict(axis),
                yaxis=dict(axis),
                zaxis=dict(axis)
            )
        )

    def set_3d_layout(self, title):
        self.set_annotations()
        self.layout = go.Layout(
            margin=dict(
                t=100
            ),
            width=1000,
            height=1000,
            showlegend=True, # mostra a legenda clicavel
           scene = dict(
               xaxis=dict(
                       autorange=True,
                       showgrid=False,
                       zeroline=False,
                       showline=False,
                       ticks='',
                       title='', # esconde o texto do eixo
                       showticklabels=False
                   ),
               yaxis=dict(
                       autorange=True,
                       showgrid=False,
                       zeroline=False,
                       showline=False,
                       ticks='',
                       title='',
                       showticklabels=False
                   ),
              zaxis=dict(
                      autorange=True,
                      showgrid=False,
                      zeroline=False,
                      showline=False,
                      ticks='',
                      title='',
                      showticklabels=False
                  ),
            aspectratio = dict(
              x = 1,
              y = 1,
              z = 1,

            ),
            camera = dict(
              center = dict(
                x = 0,
                y = 0,
                z = 0
              ),
              eye = dict(
                x = 1.96903462608,
                y = -1.09022831971,
                z = 0.405345349304
              ),
              up = dict(
                x = 0,
                y = 0,
                z = 1
              )
            ),
            dragmode = "turntable",
            annotations = self.annotations
          ),

        )

    def set_layout(self, title=''):
        if self.dim != 3:
            self.set_2d_layout(title)
        else:
            self.set_3d_layout(title)

    def get_cmin_cmax(self, attribute):
            for g in self.G.nodes:
                try:
                    if self.G.nodes[g][attribute] < self.cmin:
                        self.cmin = self.G.nodes[g][attribute]
                    if self.G.nodes[g][attribute] > self.cmax:
                        self.cmax = self.G.nodes[g][attribute]
                except:
                    raise

    def set_color_attribute(self, attribute):
        if attribute is not None:
            self.get_cmin_cmax(attribute)

        if attribute is None:
            self.settings['marker']['color'] =  self.default_color
        else:
            color = []
            for node, adjacencies in enumerate(self.G.adjacency()):
                ncolor = self.G.nodes[list(self.G.nodes.keys())[node]][attribute]
                pcolor = (ncolor - self.cmin)/(self.cmax - self.cmin)
                color.append(get_color(pcolor)[1])
            self.settings['marker']['color'] = color

    def set_hover_attribute(self, attribute):
        if attribute == None:
            return
        if iterable(attribute) and type(attribute) != str:
            attributes = attribute
        else:
            attributes = [attribute]

        for node in self.G.nodes:
            node_info = ''
            #if self.dim == 3:
            node_info = '<b>{}</b><br>'.format(node)
            if attribute == 'all_col': # get distinct attributes for each node
                attributes = list(self.G.nodes[node].keys())
            for attribute in attributes:
                if attribute == 'pos': # filter pos from hover
                    continue
                try:
                    node_info += attribute + ': ' + str(self.G.nodes[node][attribute])+'<br>'
                except:
                    pass
            self.settings['text']+=tuple([node_info])



import networkx as nx
import plotly.graph_objs as go
#from plotly.offline import download_plotlyjs, init_notebook_mode, plot, iplot
from plotly.offline import download_plotlyjs, plot

from .nodes import *
from .edges import *

#init_notebook_mode(connected=True)

class Tareco:
    def __init__(self, layout = 1, dim=2):
        self.G = nx.Graph()
        self.dim = dim
        self.node = Node(self.G, dim)
        self.edge = Edge(self.G, dim)
        self.layout = layout

    def __plot(self, filename):
        data = [self.node.settings]
        data.extend(self.edge.settings)
        fig = go.Figure(data=data,layout=self.node.layout)
        plot(fig)
        if filename is not None:
            plot(fig, filename=filename)

    def plot(self, title=' ', node_size_col=None, node_color_col=None,
             edge_color_col=None, edge_width_col =None, hover_col='all_col',
             filename=None):
        self.node.title = title
        self.node.set_pos(self.layout, self.dim)
        self.node.set_size_attribute(node_size_col, node_color_col)
        self.node.set_layout(title)
        self.node.set_color_attribute(node_color_col)
        self.node.set_hover_attribute(hover_col)
        self.edge.set_attribute(edge_color_col, edge_width_col)
        self.__plot(filename)



# -*- coding: utf-8 -*-
from setuptools import setup

setup(
    name='tareco',
    version='0.1.9',
    url='https://github.com/naruminho/tareco',
    license='MIT License',
    author='Narumi Abe and Melissa Forti',
    author_email='mail.narumi@gmail.com',
    keywords='graph networks, plotly, networkx',
    description='A tool for plotting graph networks',
    packages=['tareco'],
    install_requires=['pandas', 'plotly', 'networkx','matplotlib'],
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Scientific/Engineering :: Visualization'
    ]
)
